/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "virtualGrids.h"


Grid::Grid(int size, std::vector<int> colors, float interval, float cylinderRadius):
	mSize(size), selectedEntity(-1), mInterval(interval) {
		
	Material *mat = new Material("color0", colors[0]);
	materials.push_back(mat);
	scene->addMaterialInTheSet(mat);
		
	for(int i=1;i<colors.size();i++) {
		mat = new Material("player", colors[i]);
		scene->addMaterialInTheSet(mat);
		materials.push_back(mat);
		mat = new Material("selectedSphere", colors[i]);
		mat->setEmissionColor(-0.6);
		scene->addMaterialInTheSet(mat);
		materials.push_back(mat);		
	}
		

		
}

Grid::~Grid() {
}


int Grid::getSize() {
	return mSize;
}


void Grid::selectEntityTTT(int player, int position) {
	EntityCubeTab *mainEntities = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);	

	if(position == -1) {
		if((selectedEntity != -1)&&(mainEntities->getValue(selectedEntity)==-player)) {
			mainEntities->setValueAndMaterial(0, materials[0], selectedEntity);
			selectedEntity = -1;
		}
	} else {
		if(position != selectedEntity) {
			int playerSphere = mainEntities->getValue(position);

			if(playerSphere == 0) {
				mainEntities->setValueAndMaterial(-player, materials[2*player], position);
			}
		
			if((selectedEntity != -1)&&(mainEntities->getValue(selectedEntity)==-player)) {
				mainEntities->setValueAndMaterial(0, materials[0], selectedEntity);
			}
			
			selectedEntity = position;
	
		}
	}
}

void Grid::selectEntitySF(int player, int hit) {
	EntityPlane *verticalRods = dynamic_cast <EntityPlane*> (children[verticalRodsIndex]);	

	if(hit == -1) {
		if(selectedEntity != -1) {
			verticalRods->setValueAndMaterial(0, materials[0], selectedEntity);
			selectedEntity = -1;
		}
	} else {
		if(hit != selectedEntity) {
			
			verticalRods->setValueAndMaterial(-player, materials[2*player], hit);
			if(selectedEntity != -1) {
				verticalRods->setValueAndMaterial(0, materials[0], selectedEntity);
			} 
			selectedEntity = hit;
			
		}
	}	
}

bool Grid::changePlayerTTT(int player, int position) {
	
	bool res = false;
	EntityCubeTab *mainEntities = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);	
	
	if(position == -1) {
		if(selectedEntity != -1) {
			mainEntities->setValueAndMaterial(0, materials[0], selectedEntity);
			selectedEntity = -1;
		}
	} else {
		
		int playerSphere = mainEntities->getValue(position);

		if( playerSphere == 0) {
			mainEntities->setValueAndMaterial(-player, materials[2*player], position);
		
			if(selectedEntity != -1) {
				mainEntities->setValueAndMaterial(0, materials[0], selectedEntity);
			}
			selectedEntity = position;
		} else {
			if(playerSphere == -player) {
				mainEntities->setValueAndMaterial(player, materials[2*player-1], position);	
				selectedEntity = -1;
				res = true;
			}			
		} 	
	}

	return res;
}


bool Grid::changePlayerSF(int player, int hit, int position) {
	
	bool res = false;
	EntityPlane *verticalRods = dynamic_cast <EntityPlane*> (children[verticalRodsIndex]);	
	
	if(position == -1) {
		if(selectedEntity != -1) {
			verticalRods->setValueAndMaterial(0, materials[0], selectedEntity);
			selectedEntity = -1;
		}
	} else {
		if(verticalRods->getValue(hit) == -player) {
			verticalRods->setValueAndMaterial(0, materials[0], hit);
			selectedEntity = -1;
			res = true;
		} else {
			verticalRods->setValueAndMaterial(-player, materials[2*player], hit);
			if(selectedEntity != -1) {
				verticalRods->setValueAndMaterial(0, materials[0], selectedEntity);
			} 
			selectedEntity = hit;
		}
	}

	return res;
}


/********************************************/
/*		GridCube							*/
/********************************************/

GridCube::GridCube(int size, std::vector<int> colors, float interval, float cylinderRadius):
	Grid(size, colors, interval, cylinderRadius)
{

	cylinderDiag = new CylinderMesh (((float) mSize - 1.) * mInterval * sqrt(2.),1 , cylinderRadius, 5, false);
	scene->addMeshInTheSet(cylinderDiag, materials[1]->getNumberOfTextures());
	
	cylinderDiagCube = new CylinderMesh (((float) mSize - 1.) * mInterval * sqrt(3.),1 , cylinderRadius, 5, false);
	scene->addMeshInTheSet(cylinderDiagCube, materials[1]->getNumberOfTextures());

}

GridCube::~GridCube() {
	
}

int GridCube::getNbSpheres() {
	return mSize*mSize*mSize;
}


int GridCube::incScore(int player, int position) {
	int res = 0;	
		
	int i = position / (mSize*mSize);
	int r = position % (mSize*mSize);
	int j = r / mSize;
	int k = r % mSize;	

	res += updateAxis(player, i, j, k);		
	res += updateDiag(player, i, j, k);	
	res += updateDiagCube(player, i, j, k);	
	
	return res;
}



int GridCube::updateDiag(int player, int i, int j, int k) {
	int res = 0;
	EntityCubeTab *tab = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);		

	if(j==k){
		if(tab->testDiag(1, i, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiag, true,
															 Vector3(((float) i - ((float) mSize - 1.)/2.) * mInterval,0.,0.), 
															 Vector3(0.,-1./sqrt(2.),1./sqrt(2.)), 90.);
			addChild(cyl);
			res++;
		}
	}
	if(j+k==mSize-1){
		if(tab->testDiag(-1, i, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiag, true,
															 Vector3(((float) i - ((float) mSize - 1.)/2.) * mInterval,0.,0.), 
															 Vector3(0.,1./sqrt(2.),1./sqrt(2.)), 90.);
			addChild(cyl);
			res++;
		}
	}	
	if(k==i){
		if(tab->testDiag(2, j, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiag, true,
															 Vector3(0., ((float) j - ((float) mSize - 1.)/2.) * mInterval,0.), 
															 Vector3(0.,1.,0.), -45.);
			addChild(cyl);
			res++;
		}
	}
	if(k+i==mSize-1){
		if(tab->testDiag(-2, j, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiag, true,
															 Vector3(0., ((float) j - ((float) mSize - 1.)/2.) * mInterval,0.), 
															 Vector3(0.,1.,0.), 45.);
			addChild(cyl);
			res++;
		}
	}	
	if(i==j){
		if(tab->testDiag(3, k, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiag, true,
															 Vector3(0., 0., ((float) k - ((float) mSize - 1.)/2.) * mInterval), 
															 Vector3(0.,0.,1.), 45.);
			addChild(cyl);
			res++;
		}
	}
	if(i+j==mSize-1){
		if(tab->testDiag(-3, k, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiag, true,
															 Vector3(0., 0., ((float) k - ((float) mSize - 1.)/2.) * mInterval), 
															 Vector3(0.,0.,1.), -45.);
			addChild(cyl);
			res++;
		}
	}	
	
	return res;	
}

int GridCube::updateDiagCube(int player, int i, int j, int k) {
	int res = 0;
	EntityCubeTab *tab = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);		
	
	if((i==j)&&(j==k)){
		if(tab->testDiagCube(1, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiagCube, true,
															 Vector3(0., 0., 0.), 
															 Vector3(0.,-1./sqrt(2.),1./sqrt(2.)), ANGLE_DIAG_CUBE);
			addChild(cyl);
			res++;
		}
	}
	if((i==mSize-1-j)&&(j==k)){
		if(tab->testDiagCube(2, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiagCube, true,
															 Vector3(0., 0., 0.), 
															 Vector3(0.,-1./sqrt(2.),1./sqrt(2.)), -ANGLE_DIAG_CUBE);
			addChild(cyl);
			res++;
		}
	}			
	if((i==j)&&(j==mSize-1-k)){
		if(tab->testDiagCube(3, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiagCube, true,
															 Vector3(0., 0., 0.), 
															 Vector3(0.,1./sqrt(2.),1./sqrt(2.)), ANGLE_DIAG_CUBE);
			addChild(cyl);
			res++;
		}
	}
	if((i==mSize-1-j)&&(j==mSize-1-k)){
		if(tab->testDiagCube(4, player) == mSize){
			EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderDiagCube, true,
															 Vector3(0., 0., 0.), 
															 Vector3(0.,1./sqrt(2.),1./sqrt(2.)), -ANGLE_DIAG_CUBE);
			addChild(cyl);
			res++;
		}
	}	
	
	return res;	
}






