/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "polygon.h"

PolygonMesh::PolygonMesh(float innerRadius, int nbSides, int nbS, float texOffset) :
	Mesh("polygon", false, false, false, false), mInnerRadius(innerRadius), mNbSides(nbSides), mNbS(nbS), mTexOffset(texOffset) {
}
	
PolygonMesh::~PolygonMesh() {
}

void PolygonMesh::initFirstParams() {
	
	geometry.nbVertices = (mNbS*(mNbS + 1)/2 + 1) * mNbSides;
		
	geometry.nbIndexArrays = (mNbS-1)*mNbSides + 1;
	
	geometry.primType = new GLenum[geometry.nbIndexArrays];
	for(int i=0;i<geometry.nbIndexArrays-1;i++) {
		geometry.primType[i] = GL_TRIANGLE_STRIP;
	}
	geometry.primType[geometry.nbIndexArrays-1] = GL_TRIANGLES;
			
	geometry.nbIndices = new int[geometry.nbIndexArrays];
	for(int i=0;i<mNbS-1;i++) {
		for(int j=0;j<mNbSides;j++) {
			geometry.nbIndices[i*mNbSides+j] = 2*(mNbS-i)+1;
		}
	}	
	geometry.nbIndices[geometry.nbIndexArrays-1] = 3*mNbSides;
	
	mpUniformNormal = new Vector3(0., 0., 1.);
	mpUniformTangent = new Vector3(1., 0., 0.);
}
	
void PolygonMesh::createVertices(Vector3 *ptr) {
	float alpha = M_PI/mNbSides;
	float outerRadius = mInnerRadius / cos(alpha);
	
	float lambda;
	Vector3 a,b;
	
	float r = outerRadius;
	
	for(int i=0;i<mNbS;i++) {
		r = (1.-((float) i) /((float) mNbS)) * outerRadius;
		a = Vector3(-r * sin(alpha), -r * cos(alpha), 0.);

		for(int j=0;j<mNbSides;j++) {
			lambda = 2.*j+1.;
			b = Vector3(r * sin(lambda*alpha), -r * cos(lambda*alpha), 0.);
			
			for(int k=0;k<mNbS-i;k++) {
				lambda = (float) k / ((float) mNbS - (float) i);
				*ptr = a + lambda*(b-a);
				ptr++;
			}
			a = b;
		}
	}
	
	for(int j=0;j<mNbSides;j++) {
		*ptr = Vector3(0.,0.,0.);
		ptr++;
	}
}

void PolygonMesh::createColors(Color *ptr) {
	
}

void PolygonMesh::createNormals(Vector3 *ptr) {
	//uniform normal
}

void PolygonMesh::createTexCoords(GLfloat *ptr) {
	float alpha = M_PI/mNbSides;
	float r = 1./cos(alpha);
	float lambda;
	float as,at,bs,bt;
				
	
	for(int i=0;i<mNbS;i++) {
		r = (1.-mTexOffset)*(1.-(((float) i) /((float) mNbS))) / cos(alpha);
		as = (1.-r*sin(alpha))/2.;
		at = (1.-r*cos(alpha))/2.;
		
		for(int j=0;j<mNbSides;j++) {
			
			lambda = 2.*j+1.;
			bs = (1.+ r*sin(lambda*alpha))/2.;
			bt = (1.- r*cos(lambda*alpha))/2.;
			
			for(int k=0;k<mNbS-i;k++) {
				lambda = (float) k / ((float) mNbS - (float) i);
				*ptr = as + lambda * (bs-as);
				ptr++;
				*ptr = at + lambda * (bt-at);
				ptr++;				
			}
			as = bs;
			at = bt;			
		}
	}
	
	for(int j=0;j<mNbSides;j++) {
		*ptr = 0.5;
		ptr++;
		*ptr = 0.5;
		ptr++;
	}	
}

void PolygonMesh::createFogCoords(GLfloat *ptr) {
	
}
	
void PolygonMesh::createTangents(Vector3 *ptr) {
	float alpha = M_PI/mNbSides;
	
	float lambda;
	Vector3 a,b,tangent;
		
	for(int i=0;i<mNbS;i++) {
		a = Vector3(-sin(alpha), -cos(alpha), 0.);

		for(int j=0;j<mNbSides;j++) {
			lambda = 2.*j+1.;
			b = Vector3(sin(lambda*alpha), -cos(lambda*alpha), 0.);
			
			for(int k=0;k<mNbS-i;k++) {
				lambda = (float) k / ((float) mNbS - (float) i);
				tangent = a + lambda*(b-a);
				tangent.normalize();
				*ptr = tangent;
				ptr++;
			}
			a = b;
		}
	}
	
	for(int j=0;j<mNbSides;j++) {
		lambda = 2.*j;
		*ptr = Vector3(sin(lambda*alpha), -cos(lambda*alpha), 0.);
		ptr++;
	}	
}

void PolygonMesh::createIndices(int numArray, GLuint *ptr) {
		
	if(numArray == (mNbS-1)*mNbSides) {		
		//triangles : polygon center
		for(int j=0;j<mNbSides-1;j++) {
			*ptr = (mNbS+2)*(mNbS-1)/2 * mNbSides + j;
			ptr++;
			*ptr = (mNbS+2)*(mNbS-1)/2 * mNbSides + j+1;
			ptr++;
			*ptr = (mNbS+1)*mNbS/2 * mNbSides + j;
			ptr++;			
		}
		*ptr = (mNbS+2)*(mNbS-1)/2 * mNbSides + mNbSides-1;
		ptr++;
		*ptr = (mNbS+2)*(mNbS-1)/2 * mNbSides;
		ptr++;
		*ptr = (mNbS+1)*mNbS/2 * mNbSides + mNbSides-1;
		ptr++;	
		
	} else {	
		
		//we have numArray = i * mNbSides +j
		int i = numArray / mNbSides;
		int j = numArray % mNbSides;
		int k;
				
		int startLine = (2*mNbS -i + 1)* i/2 * mNbSides;
		int startNextLine = (2*mNbS -i)* (i+1)/2 * mNbSides;		
		
		for(k=0;k<mNbS-i-1;k++) {
			*ptr = startLine + j*(mNbS-i) + k;
			ptr++;
			*ptr = startNextLine + j*(mNbS-i-1) + k;
			ptr++;
		}
		
		k = mNbS-i-1;
		*ptr = startLine + j*(mNbS-i) + k;
		ptr++;		
		
		if(j!=mNbSides-1) {
			*ptr = startNextLine + j*(mNbS-i-1) + k;
			ptr++;
			*ptr = startLine + j*(mNbS-i) + k+1;
			ptr++;			
		} else {
			*ptr = startNextLine;
			ptr++;
			*ptr = startLine;
			ptr++;	
		}	
	}	
}


